@@ -1,14 +1,34 @@
 /*
- * Plumbing.java
+ * java-gnome, a UI library for writing GTK and GNOME programs from Java!
  *
- * Copyright (c) 2007-2008 Operational Dynamics Consulting Pty Ltd
- * 
- * The code in this file, and the library it is a part of, are made available
- * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" plus the "Classpath Exception" (you may link to this code as a
- * library into other programs provided you don't make a derivation of it).
- * See the LICENCE file for the terms governing usage and redistribution.
-
+ * Copyright © 2007-2010 Operational Dynamics Consulting, Pty Ltd
+ *
+ * The code in this file, and the program it is a part of, is made available
+ * to you by its authors as open source software: you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License version
+ * 2 ("GPL") as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GPL for more details.
+ *
+ * You should have received a copy of the GPL along with this program. If not,
+ * see http://www.gnu.org/licenses/. The authors of this program may be
+ * contacted through http://java-gnome.sourceforge.net/.
+ *
+ * Linking this library statically or dynamically with other modules is making
+ * a combined work based on this library. Thus, the terms and conditions of
+ * the GPL cover the whole combination. As a special exception (the
+ * "Claspath Exception"), the copyright holders of this library give you
+ * permission to link this library with independent modules to produce an
+ * executable, regardless of the license terms of these independent modules,
+ * and to copy and distribute the resulting executable under terms of your
+ * choice, provided that you also meet, for each linked independent module,
+ * the terms and conditions of the license of that module. An independent
+ * module is a module which is not derived from or based on this library. If
+ * you modify this library, you may extend the Classpath Exception to your
+ * version of the library, but you are not obligated to do so. If you do not
+ * wish to do so, delete this exception statement from your version.
  */
 package org.gnome.gdk;
 
@@ -25,6 +45,8 @@ public abstract class Plumbing extends org.gnome.glib.Plumbing
     protected Plumbing() {}
 
     static {
+        isLibraryReady();
+
         /*
          * When you call getScreen(), the object returned appears to be a
          * sublcass of GdkScreen, in this case GdkScreenX11. It doesn't seem
@@ -34,6 +56,7 @@ public abstract class Plumbing extends org.gnome.glib.Plumbing
          * back that are subtypes we don't know about.
          */
         registerType("GdkScreenX11", org.gnome.gdk.Screen.class);
+        registerType("GdkDisplayX11", org.gnome.gdk.Display.class);
     }
 
     /**
@@ -129,6 +152,10 @@ public abstract class Plumbing extends org.gnome.glib.Plumbing
                 type = EventProximity.class;
                 break;
 
+            case GdkEventType.OWNER_CHANGE:
+                type = EventOwnerChange.class;
+                break;
+
             case GdkEventType.WINDOW_STATE:
                 type = EventWindowState.class;
                 break;
@@ -149,7 +176,7 @@ public abstract class Plumbing extends org.gnome.glib.Plumbing
         }
 
         proxy = org.gnome.glib.Plumbing.boxedFor(type, pointer);
-        return (Boxed) proxy;
+        return proxy;
     }
 
     private static native final int getEventTypeOrdinal(long pointer);
