@@ -1,26 +1,48 @@
 /*
- * CellRenderer.java
+ * java-gnome, a UI library for writing GTK and GNOME programs from Java!
  *
- * Copyright (c) 2007 Operational Dynamics Consulting Pty Ltd
+ * Copyright © 2007-2010 Operational Dynamics Consulting, Pty Ltd and Others
  *
- * The code in this file, and the library it is a part of, are made available
- * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" plus the "Classpath Exception" (you may link to this code as a
- * library into other programs provided you don't make a derivation of it).
- * See the LICENCE file for the terms governing usage and redistribution.
+ * The code in this file, and the program it is a part of, is made available
+ * to you by its authors as open source software: you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License version
+ * 2 ("GPL") as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GPL for more details.
+ *
+ * You should have received a copy of the GPL along with this program. If not,
+ * see http://www.gnu.org/licenses/. The authors of this program may be
+ * contacted through http://java-gnome.sourceforge.net/.
+ *
+ * Linking this library statically or dynamically with other modules is making
+ * a combined work based on this library. Thus, the terms and conditions of
+ * the GPL cover the whole combination. As a special exception (the
+ * "Claspath Exception"), the copyright holders of this library give you
+ * permission to link this library with independent modules to produce an
+ * executable, regardless of the license terms of these independent modules,
+ * and to copy and distribute the resulting executable under terms of your
+ * choice, provided that you also meet, for each linked independent module,
+ * the terms and conditions of the license of that module. An independent
+ * module is a module which is not derived from or based on this library. If
+ * you modify this library, you may extend the Classpath Exception to your
+ * version of the library, but you are not obligated to do so. If you do not
+ * wish to do so, delete this exception statement from your version.
  */
 package org.gnome.gtk;
 
 /**
- * 
+ * Present data in CellLayouts (ie, the TreeViewColumns of TreeView, and
+ * ComboBox).
  * 
  * <p>
- * Note that there is not one CellRenderer per cell in the table like
- * presentation of a TreeView! Instead, a CellRenderer is an engine called
- * upon to draw <i>many</i> cells. The Widget employing them will, by turn,
- * set various properties (the actual data for the current cell in question
- * would be what changes, though most of the rest of the properties would
- * likely remain constant) and then ask the CellRenderer to return the
+ * Note that there is not one CellRenderer per individual cell in the
+ * table-like presentation of a TreeView! Instead, a CellRenderer is an engine
+ * called upon to draw <i>many</i> cells. The Widget employing them will, by
+ * turn, set various properties (the actual data for the current cell in
+ * question would be what changes, though most of the rest of the properties
+ * would likely remain constant) and then ask the CellRenderer to return the
  * rendered result.
  * 
  * <p>
@@ -32,9 +54,9 @@ package org.gnome.gtk;
  * <p>
  * While the whole point of the CellRenderers is to present the data from a
  * given column of your data model, CellRenderers also offer a considerable
- * number of properties that you <i>don't</i> need to vary row by row but
- * want to set for the TreeViewColumn as a whole. For these you will notice a
- * a setter which takes a conventional type as its argument; setting this
+ * number of properties that you <i>don't</i> need to vary row by row but want
+ * to set for the TreeViewColumn as a whole. For these you will notice a a
+ * setter which takes a conventional type as its argument; setting this
  * property will set it for all rows in that column. (Occasionally you will
  * see the setter for a property overloaded to offer you both a DataColumn
  * driven mode and a fixed value mode for the rare occasions where both styles
@@ -43,15 +65,15 @@ package org.gnome.gtk;
  * 
  * <p>
  * It's easy to be tempted into thinking that CellRenderers are Widgets,
- * especially as they get <code>pack()</code>ed into TreeViewColumns on
- * their way to being used in TreeViews. They are, however, merely utility
- * elements that are used to facilitate drawing, and <i>not</i> full power
- * Widgets.
+ * especially as they get <code>pack()</code>ed into TreeViewColumns on their
+ * way to being used in TreeViews. They are, however, merely utility elements
+ * that are used to facilitate drawing, and <i>not</i> full power Widgets.
  * 
  * <p>
  * Do not try to reuse a CellRenderer between different TreeViewColumns.
  * 
  * @author Andrew Cowie
+ * @author Stefan Prelle
  * @since 4.0.5
  */
 /*
@@ -81,12 +103,7 @@ public abstract class CellRenderer extends Object
      * <p>
      * In case you hadn't noticed yet, TreeViewColumn is a CellLayout.
      */
-    /*
-     * Pack into the TreeViewColumn with expand true; at the moment we're only
-     * exposing one CellRenderer per column, so might as well give it the
-     * space. Testing showed this to make sense at runtime.
-     */
-    protected CellRenderer(long pointer, CellLayout vertical) {
+    protected CellRenderer(long pointer, CellLayout vertical, boolean expand) {
         super(pointer);
 
         if (vertical == null) {
@@ -94,7 +111,7 @@ public abstract class CellRenderer extends Object
                     "Must pass an instantiated TreeViewColumn to the CellRenderer constructor");
         }
 
-        GtkCellLayout.packStart(vertical, this, true);
+        GtkCellLayout.packStart(vertical, this, expand);
 
         this.vertical = vertical;
     }
@@ -137,4 +154,26 @@ public abstract class CellRenderer extends Object
     public void setBackground(String colour) {
         setPropertyString("cell-background", colour);
     }
+
+    /**
+     * Indicate if the CellRenderer should be visible, depending on the
+     * content of a DataColumnBoolean. For example, you could set use this
+     * property to control whether or not a {@link CellRendererToggle} was
+     * visible in a TreeView where you want each of the leaf rows to have such
+     * a Button but you do not want the parent node rows to have one.
+     * 
+     * @since 4.0.9
+     */
+    public void setVisible(DataColumnBoolean column) {
+        GtkCellLayout.addAttribute(vertical, this, "visible", column.getOrdinal());
+    }
+
+    /**
+     * Fix the width of this CellRenderer, specified in pixels.
+     * 
+     * @since 4.0.20
+     */
+    public void setWidth(int width) {
+        setPropertyInteger("width", width);
+    }
 }
