@@ -1,13 +1,34 @@
 /*
- * TextView.java
+ * java-gnome, a UI library for writing GTK and GNOME programs from Java!
  *
- * Copyright (c) 2007-2009 Operational Dynamics Consulting Pty Ltd, and Others
+ * Copyright © 2007-2011 Operational Dynamics Consulting, Pty Ltd and Others
  *
- * The code in this file, and the library it is a part of, are made available
- * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" plus the "Classpath Exception" (you may link to this code as a
- * library into other programs provided you don't make a derivation of it).
- * See the LICENCE file for the terms governing usage and redistribution.
+ * The code in this file, and the program it is a part of, is made available
+ * to you by its authors as open source software: you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License version
+ * 2 ("GPL") as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GPL for more details.
+ *
+ * You should have received a copy of the GPL along with this program. If not,
+ * see http://www.gnu.org/licenses/. The authors of this program may be
+ * contacted through http://java-gnome.sourceforge.net/.
+ *
+ * Linking this library statically or dynamically with other modules is making
+ * a combined work based on this library. Thus, the terms and conditions of
+ * the GPL cover the whole combination. As a special exception (the
+ * "Claspath Exception"), the copyright holders of this library give you
+ * permission to link this library with independent modules to produce an
+ * executable, regardless of the license terms of these independent modules,
+ * and to copy and distribute the resulting executable under terms of your
+ * choice, provided that you also meet, for each linked independent module,
+ * the terms and conditions of the license of that module. An independent
+ * module is a module which is not derived from or based on this library. If
+ * you modify this library, you may extend the Classpath Exception to your
+ * version of the library, but you are not obligated to do so. If you do not
+ * wish to do so, delete this exception statement from your version.
  */
 package org.gnome.gtk;
 
@@ -178,12 +199,17 @@ import org.gnome.pango.FontDescription;
  * and so on.
  * 
  * <p>
+ * A possible alternative to the above workaround is to iterate the main loop;
+ * See {@link Gtk#mainIterationDo(boolean) Gtk.mainIterationDo()}.
+ * 
+ * <p>
  * <i>Obviously "internal to GTK" implies that we are second guessing the
  * implementation details. This workaround is not based on documented public
  * behaviour, and unfortunately is not guaranteed to be stable. So as we say
  * in Open Source, Your Mileage May Vary. Perhaps GTK will improve this aspect
  * of the library in the future.</i>
  * 
+ * <a name="spelling"></a>
  * <h2>Spellchecking</h2>
  * 
  * <img class="snapshot" src="TextView-Spelling.png"> You can add
@@ -991,12 +1017,10 @@ public class TextView extends Container
     }
 
     /**
-     * Set the number of pixels that will be between the left hand edge of the
-     * TextView and the left hand edge of the paragraphs of text.
-     * 
-     * @since 4.0.10
+     * @deprecated
      */
     public void setLeftMargin(int pixels) {
+        assert false : "use setMarginLeft() instead";
         GtkTextView.setLeftMargin(this, pixels);
     }
 
@@ -1059,4 +1083,16 @@ public class TextView extends Container
         }
         return spell;
     }
+
+    /**
+     * Tell this TextView to adopt the given justification.
+     * 
+     * @since 4.0.14
+     */
+    /*
+     * Method name adjusted to match Label's setJustify().
+     */
+    public void setJustify(Justification setting) {
+        GtkTextView.setJustification(this, setting);
+    }
 }
