@@ -1,16 +1,39 @@
 /*
- * TreeView.java
+ * java-gnome, a UI library for writing GTK and GNOME programs from Java!
  *
- * Copyright (c) 2007-2008 Operational Dynamics Consulting Pty Ltd, and Others
+ * Copyright © 2007-2010 Operational Dynamics Consulting, Pty Ltd and Others
  *
- * The code in this file, and the library it is a part of, are made available
- * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" plus the "Classpath Exception" (you may link to this code as a
- * library into other programs provided you don't make a derivation of it).
- * See the LICENCE file for the terms governing usage and redistribution.
+ * The code in this file, and the program it is a part of, is made available
+ * to you by its authors as open source software: you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License version
+ * 2 ("GPL") as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GPL for more details.
+ *
+ * You should have received a copy of the GPL along with this program. If not,
+ * see http://www.gnu.org/licenses/. The authors of this program may be
+ * contacted through http://java-gnome.sourceforge.net/.
+ *
+ * Linking this library statically or dynamically with other modules is making
+ * a combined work based on this library. Thus, the terms and conditions of
+ * the GPL cover the whole combination. As a special exception (the
+ * "Claspath Exception"), the copyright holders of this library give you
+ * permission to link this library with independent modules to produce an
+ * executable, regardless of the license terms of these independent modules,
+ * and to copy and distribute the resulting executable under terms of your
+ * choice, provided that you also meet, for each linked independent module,
+ * the terms and conditions of the license of that module. An independent
+ * module is a module which is not derived from or based on this library. If
+ * you modify this library, you may extend the Classpath Exception to your
+ * version of the library, but you are not obligated to do so. If you do not
+ * wish to do so, delete this exception statement from your version.
  */
 package org.gnome.gtk;
 
+import org.gnome.gdk.Rectangle;
+
 /**
  * Display the data from a {@link TreeModel} in a tabular form. TreeViews are
  * ubiquitous in most applications, being used to both output data in list
@@ -88,12 +111,12 @@ package org.gnome.gtk;
  * Dealing with the events generated on the TreeView is either straight
  * forward or quite complicated, depending on what you are trying to
  * accomplish. If you just need a callback when the user activates a row in
- * the display, then the {@link TreeView.ROW_ACTIVATED ROW_ACTIVATED} signal
- * will do the trick fairly simply; see its documentation for an example. For
- * anything else, you will need to use the {@link TreeSelection TreeSelection}
- * helper class (every TreeView automatically has one). It has a
- * {@link TreeSelection.CHANGED CHANGED} signal which you hook up to which
- * will tell you what row(s) are currently selected.
+ * the display, then the {@link TreeView.RowActivated} signal will do the
+ * trick fairly simply; see its documentation for an example. For anything
+ * else, you will need to use the {@link TreeSelection TreeSelection} helper
+ * class (every TreeView automatically has one). It has a
+ * {@link TreeSelection.Changed} signal which you hook up to which will tell
+ * you what row(s) are currently selected.
  * 
  * <pre>
  * selection = view.getSelection();
@@ -114,6 +137,7 @@ package org.gnome.gtk;
  * 
  * @author Andrew Cowie
  * @author Srichand Pendyala
+ * @author Vreixo Formoso
  * @since 4.0.5
  */
 public class TreeView extends Container
@@ -180,6 +204,13 @@ public class TreeView extends Container
         return vertical;
     }
 
+    /**
+     * Remove a tree view column from this TreeView.
+     */
+    public void removeColumn(TreeViewColumn column) {
+        GtkTreeView.removeColumn(this, column);
+    }
+
     /**
      * Set whether this TreeView has a header row at the top of the Widget
      * showing the titles of each of the TreeViewColumns packed into it. The
@@ -192,9 +223,9 @@ public class TreeView extends Container
     /**
      * Set whether the column titles in the header row can be clicked to
      * change the sorting of the displayed data. While the default is
-     * <code>false</code> (since you frequently have the rows ordered the
-     * way they are for a reason and don't want to let the user be reordering
-     * the display and getting lost in the process), calling TreeViewColumn's
+     * <code>false</code> (since you frequently have the rows ordered the way
+     * they are for a reason and don't want to let the user be reordering the
+     * display and getting lost in the process), calling TreeViewColumn's
      * {@link TreeViewColumn#setSortColumn(DataColumn) setSortColumn()} will
      * make the headers clickable. Use this method after your column setup to
      * turn it off [again].
@@ -205,19 +236,19 @@ public class TreeView extends Container
 
     /**
      * Emitted when a row in the TreeView has been activated. Activation
-     * occurs when a row in the view is double-clicked, or when
-     * <code>Space</code> or <code>Enter</code> is pressed while a row is
-     * selected.
+     * occurs when a row in the view is double-clicked, or when <b>
+     * <code>Space</code></b> or <b><code>Enter</code></b> are pressed while a
+     * row is selected.
      * 
      * <p>
      * In general, you've got the TreeModel and especially its DataColumns
-     * visible, so to use <code>ROW_ACTIVATED</code> you can just:
+     * visible, so to use <code>TreeView.RowActivated</code> you can just:
      * 
      * <pre>
      * final TreeModel model;
      * final DataColumnString column;
      * 
-     * view.connect(new TreeView.ROW_ACTIVATED() {
+     * view.connect(new TreeView.RowActivated() {
      *     public void onRowActivated(TreeView source, TreePath path, TreeViewColumn vertical) {
      *         final TreeIter row;
      * 
@@ -232,34 +263,193 @@ public class TreeView extends Container
      * the model, so get on with using <code>path</code> right away.
      * 
      * <p>
-     * <code>ROW_ACTIVATED</code> is perfectly sufficient for basic
+     * <code>TreeView.RowActivated</code> is perfectly sufficient for basic
      * situations, but you may need to see TreeSelection's
-     * {@link TreeSelection.CHANGED CHANGED} to for more complicated selection
-     * and activation expressions. In practise you'll use both.
+     * {@link TreeSelection.Changed} to for more complicated selection and
+     * activation expressions. In practise you'll use both.
      * 
      * @author Andrew Cowie
      * @since 4.0.5
      */
-    public interface ROW_ACTIVATED extends GtkTreeView.ROW_ACTIVATED
+    public interface RowActivated extends GtkTreeView.RowActivatedSignal
     {
         /**
          * The useful parameter is usually <code>path</code> which can be
          * converted into a TreeIter with your TreeModel's
          * {@link TreeModel#getIter(TreePath) getIter()} allowing you to then
          * lookup a particular value from the data model. You rarely need
-         * <code>vertical</code> but it can give you some indication in
-         * which column the click happened.
+         * <code>vertical</code> but it can give you some indication in which
+         * column the click happened.
          */
         public void onRowActivated(TreeView source, TreePath path, TreeViewColumn vertical);
     }
 
     /**
-     * Hook up a <code>ROW_ACTIVATED</code> handler.
+     * Hook up a <code>TreeView.RowActivated</code> handler.
      */
-    public void connect(ROW_ACTIVATED handler) {
-        GtkTreeView.connect(this, handler);
+    public void connect(TreeView.RowActivated handler) {
+        GtkTreeView.connect(this, handler, false);
     }
 
+    /**
+     * Cause a <code>TreeView.RowActivated</code> signal to be emitted for the
+     * given TreePath. The TreeViewColumn is optional; use <code>null</code>
+     * if you don't want to specify it.
+     * 
+     * @since 4.0.9
+     */
+    public void emitRowActivated(TreePath path, TreeViewColumn vertical) {
+        GtkTreeView.rowActivated(this, path, vertical);
+    }
+
+    /** @deprecated */
+    public interface ROW_ACTIVATED extends GtkTreeView.RowActivatedSignal
+    {
+    }
+
+    /** @deprecated */
+    public void connect(ROW_ACTIVATED handler) {
+        assert false : "use TreeView.RowActivated instead";
+        GtkTreeView.connect(this, handler, false);
+    }
+
+    /**
+     * Emitted when a row in the TreeView has been expanded, i.e. when its
+     * child nodes are shown. A row is expanded either by clicking in the
+     * little arrow near it, or by pressing the <code>+</code> key when a row
+     * is selected. Of course, a row can be only expanded when it has child
+     * rows, and so it can be only emitted when the TreeView is used with a
+     * hierarchical model such as {@link TreeStore}.
+     * 
+     * <p>
+     * In general, you've got the TreeModel and especially its DataColumns
+     * visible, so to use <code>TreeView.RowExpanded</code> you can just:
+     * 
+     * <pre>
+     * final TreeModel model;
+     * final DataColumnString column;
+     * 
+     * view.connect(new TreeView.RowExpanded() {
+     *     public void onRowExpanded(TreeView source, TreeIter iter, TreePath path) {
+     *         ... = model.getValue(iter, column);
+     *     }
+     * });
+     * </pre>
+     * 
+     * Remember that TreeIters and TreePaths are not stable over changes to
+     * the model, so get on with using <code>path</code> right away.
+     * 
+     * @author Vreixo Formoso
+     * @since 4.0.7
+     */
+    public interface RowExpanded
+    {
+        public void onRowExpanded(TreeView source, TreeIter iter, TreePath path);
+    }
+
+    /**
+     * Hook up a <code>TreeView.RowExpanded</code> handler.
+     * 
+     * @since 4.0.7
+     */
+    public void connect(TreeView.RowExpanded handler) {
+        GtkTreeView.connect(this, new RowExpandedHandler(handler), false);
+    }
+
+    /*
+     * This internal class is needed because the TreeIter passed to the
+     * handler does not have the model field properly set, so we need to set
+     * it before passing the TreeIter to the user.
+     */
+    private static class RowExpandedHandler implements GtkTreeView.RowExpandedSignal
+    {
+        private final TreeView.RowExpanded handler;
+
+        private RowExpandedHandler(TreeView.RowExpanded handler) {
+            super();
+            this.handler = handler;
+        }
+
+        public void onRowExpanded(TreeView source, TreeIter iter, TreePath path) {
+            iter.setModel(source.getModel());
+            handler.onRowExpanded(source, iter, path);
+        }
+    }
+
+    /** @deprecated */
+    public interface ROW_EXPANDED extends GtkTreeView.RowExpandedSignal
+    {
+    }
+
+    /** @deprecated */
+    public void connect(ROW_EXPANDED handler) {
+        assert false : "use TreeView.RowExpanded instead";
+        GtkTreeView.connect(this, new RowExpandedHandler0(handler), false);
+    }
+
+    /** @deprecated */
+    private static class RowExpandedHandler0 implements GtkTreeView.RowExpandedSignal
+    {
+        private final ROW_EXPANDED handler;
+
+        /** @deprecated */
+        private RowExpandedHandler0(ROW_EXPANDED handler) {
+            super();
+            this.handler = handler;
+        }
+
+        public void onRowExpanded(TreeView source, TreeIter iter, TreePath path) {
+            iter.setModel(source.getModel());
+            handler.onRowExpanded(source, iter, path);
+        }
+    }
+
+    /**
+     * Check whether the given row is expanded, i.e. whether its children are
+     * shown.
+     * 
+     * @param path
+     *            The row we want to check.
+     * @return <code>true</code> if the row is expanded, <code>false</code> if
+     *         not.
+     * @since 4.0.7
+     */
+    public boolean rowExpanded(TreePath path) {
+        return GtkTreeView.rowExpanded(this, path);
+    }
+
+    /**
+     * Expand the given row, making its children visible to the user. This has
+     * no effect if the row has no child nodes. Of course, this is always the
+     * case if you use a ListStore model.
+     * 
+     * @param path
+     *            The row to expand.
+     * @param openAll
+     *            <code>true</code> to recursively expand all children,
+     *            <code>false</code> to expand only the given row.
+     * @return <code>true</code> if the path refers to a valid row, and it has
+     *         child nodes. <code>false</code> otherwise.
+     * @since 4.0.7
+     */
+    public boolean expandRow(TreePath path, boolean openAll) {
+        return GtkTreeView.expandRow(this, path, openAll);
+    }
+
+    /**
+     * Collapse the given row, thus hiding its children if the row was
+     * previously expanded.
+     * 
+     * @param path
+     *            The row to collapse.
+     * @since 4.0.7
+     */
+    public void collapseRow(TreePath path) {
+        GtkTreeView.collapseRow(this, path);
+    }
+
+    private TreeSelection selection;
+
     /**
      * Get the TreeSelection object corresponding to this TreeView. Every
      * TreeView has a TreeSelection which is a utility instance allowing you
@@ -267,7 +457,10 @@ public class TreeView extends Container
      * method gives you access to it.
      */
     public TreeSelection getSelection() {
-        return GtkTreeView.getSelection(this);
+        if (selection == null) {
+            selection = GtkTreeView.getSelection(this);
+        }
+        return selection;
     }
 
     /**
@@ -411,8 +604,8 @@ public class TreeView extends Container
      * application.
      * 
      * <p>
-     * To fetch the current height mode, see
-     * {@link #getFixedHeightMode() getFixedHeightMode()}.
+     * To fetch the current height mode, see {@link #getFixedHeightMode()
+     * getFixedHeightMode()}.
      * 
      * @param enable
      *            <code>true</code> if all rows in the TreeView are to be of
@@ -457,16 +650,27 @@ public class TreeView extends Container
      * @author Srichand Pendyala
      * 
      */
-    public interface SELECT_ALL extends GtkTreeView.SELECT_ALL
+    public interface SelectAll extends GtkTreeView.SelectAllSignal
     {
         public boolean onSelectAll(TreeView source);
     }
 
     /**
-     * Hook up a <code>SELECT_ALL</code> signal handler.
+     * Hook up a <code>TreeView.SelectAll</code> signal handler.
      */
+    public void connect(TreeView.SelectAll handler) {
+        GtkTreeView.connect(this, handler, false);
+    }
+
+    /** @deprecated */
+    public interface SELECT_ALL extends GtkTreeView.SelectAllSignal
+    {
+    }
+
+    /** @deprecated */
     public void connect(SELECT_ALL handler) {
-        GtkTreeView.connect(this, handler);
+        assert false : "use TreeView.SelectAll instead";
+        GtkTreeView.connect(this, handler, false);
     }
 
     /**
@@ -478,8 +682,7 @@ public class TreeView extends Container
      * request.
      * 
      * <p>
-     * The default is <code>false</code>, not drawing alternating row
-     * colours.
+     * The default is <code>false</code>, not drawing alternating row colours.
      */
     public void setRulesHint(boolean setting) {
         GtkTreeView.setRulesHint(this, setting);
@@ -504,13 +707,13 @@ public class TreeView extends Container
      * @param rowAlign
      *            Determines where in the view the row specified by
      *            <code>path</code> is placed, with <code>0.0f</code>
-     *            representing top, and <code>1.0f</code> representing
-     *            bottom, as usual. The constants in Alignment such as
+     *            representing top, and <code>1.0f</code> representing bottom,
+     *            as usual. The constants in Alignment such as
      *            {@link Alignment#CENTER CENTER} can be used.
      * @param colAlign
      *            Determines where in the view the column specified by
-     *            <code>vertical</code> will be placed; <code>0.0f</code>
-     *            is fully left, <code>1.0f</code> is fully right.
+     *            <code>vertical</code> will be placed; <code>0.0f</code> is
+     *            fully left, <code>1.0f</code> is fully right.
      * @since 4.0.6
      */
     public void scrollToCell(TreePath path, TreeViewColumn vertical, float rowAlign, float colAlign) {
@@ -519,17 +722,16 @@ public class TreeView extends Container
 
     /**
      * Scroll the TreeView so that the cell specified by <code>path</code>,
-     * <code>vertical</code> is visible. This variant ignores alignment
-     * values and just scrolls the TreeView so that the cell specified is
-     * visible, closest to whichever edge it came in from, and doing nothing
-     * if the cell is already on screen.
+     * <code>vertical</code> is visible. This variant ignores alignment values
+     * and just scrolls the TreeView so that the cell specified is visible,
+     * closest to whichever edge it came in from, and doing nothing if the
+     * cell is already on screen.
      * 
      * <p>
-     * See the discussion about <code>path</code> or <code>vertical</code>
-     * in the other
-     * {@link #scrollToCell(TreePath, TreeViewColumn, float, float) scrollToCell()}
-     * method to learn how you can scroll in a single direction only if
-     * desired.
+     * See the discussion about <code>path</code> or <code>vertical</code> in
+     * the other {@link #scrollToCell(TreePath, TreeViewColumn, float, float)
+     * scrollToCell()} method to learn how you can scroll in a single
+     * direction only if desired.
      * 
      * @since 4.0.6
      */
@@ -548,7 +750,7 @@ public class TreeView extends Container
      * 
      * <p>
      * Incidentally, you can observe these changes by connecting to
-     * <code>ROW_INSERTED</code> and <code>ROW_DELETED</code>.
+     * <code>TreeView.RowInserted</code> and <code>TreeView.RowDeleted</code>.
      * 
      * @since 4.0.6
      */
@@ -560,4 +762,206 @@ public class TreeView extends Container
     public void setReorderable(boolean setting) {
         GtkTreeView.setReorderable(this, setting);
     }
+
+    /**
+     * Expand all the rows in this TreeStore backed TreeView, making
+     * <i>all</i> children visible.
+     * 
+     * @since 4.0.7
+     */
+    public void expandAll() {
+        GtkTreeView.expandAll(this);
+    }
+
+    /**
+     * Collapse all the (child) rows in this TreeStore backed TreeView. Only
+     * top level rows will be visible.
+     * 
+     * @since 4.0.7
+     */
+    public void collapseAll() {
+        GtkTreeView.collapseAll(this);
+    }
+
+    /**
+     * Get all the TreeViewColumns for this TreeView.
+     * 
+     * @since 4.0.11
+     */
+    public TreeViewColumn[] getColumns() {
+        return GtkTreeView.getColumns(this);
+    }
+
+    /**
+     * Get the TreeViewColumn at the given position in the TreeView, with
+     * <code>0</code> being the left-most one.
+     * 
+     * <p>
+     * If they know they are going to need it later, most people just keep a
+     * reference to the TreeViewColumn around when they create it.
+     * 
+     * @since 4.0.8
+     */
+    public TreeViewColumn getColumn(int index) {
+        return GtkTreeView.getColumn(this, index);
+    }
+
+    /**
+     * Set the current keyboard focus to be at a specific path in the
+     * TreeView. This method selects the row at <code>path</code>, and as a
+     * result is often used to draw a user's attention to a particular place.
+     * 
+     * <p>
+     * If <code>vertical</code> is supplied, the specific TreeViewColumn
+     * indicated is selected. This is usually used in concert with setting
+     * <code>startEditing</code> to <code>true</code> which causes the
+     * TreeView to immediately start editing at the the specified row and
+     * column (assuming, of course, that that CellRenderer has been made
+     * mutable. See {@link CellRendererText#setEditable(boolean)
+     * setEditable()}).
+     * 
+     * @since 4.0.8
+     */
+    public void setCursor(TreePath path, TreeViewColumn vertical, boolean startEditing) {
+        GtkTreeView.setCursor(this, path, vertical, startEditing);
+    }
+
+    /**
+     * Get a TreePath indicating what row in the TreeView a given set of
+     * co-ordinates corresponds to.
+     * 
+     * <p>
+     * The position indicated by (<code>x</code>,<code>y</code>) is in
+     * <var>bin co-ordinates</var>. Usually you are working in the context of
+     * a handler hooked up to an Event and these values should be obtained
+     * from that Event.
+     * 
+     * <p>
+     * See also {@link #getColumnAtPos(int, int) getColumnAtPos()} for the
+     * complementary method to find out what vertical the co-ordinates
+     * correspond to.
+     * 
+     * <h2>Handling right-clicks</h2>
+     * 
+     * <p>
+     * It is common to create a context menu as a result of a right-click on a
+     * TreeView. Ordinarily, you would intercept the
+     * <code>Widget.ButtonPressEvent</code> signal and then prepare your Menu
+     * in the handler there, quite reasonably expecting that the row that you
+     * have right-clicked on would be selected. Unfortunately, if you hook up
+     * to that signal your code will run before the default handler and the
+     * <i>previously</i> selected row will still be selected while your
+     * handler runs. This is annoying. It is the default
+     * <code>Widget.ButtonPressEvent</code> handler which selects the new row,
+     * so you have to manually select it yourself before acting on the
+     * right-click. It is <code>getPathAtPos()</code> which gives you the
+     * ability to do so:
+     * 
+     * <pre>
+     * view.connect(new Widget.ButtonPressEvent() {
+     *     public boolean onButtonPressEvent(Widget source, EventButton event) {
+     *         final int x, y;
+     *         final TreePath path;
+     *         final TreeSelection selection;
+     * 
+     *         if (event.getButton() != MouseButton.RIGHT) {
+     *             return false;
+     *         }
+     * 
+     *         x = (int) event.getX();
+     *         y = (int) event.getY();
+     *         path = view.getPathAtPos(x, y);
+     * 
+     *         selection = view.getSelection();
+     *         selection.selectRow(path);
+     * 
+     *         // and now pop your context menu, doing
+     *         // something with the row as appropriate.
+     * 
+     *         return true;
+     *     }
+     * });
+     * </pre>
+     * 
+     * @return Will return <code>null</code> if the passed in co-ordinates do
+     *         not correspond to a row in the TreeView.
+     * @since 4.0.9
+     */
+    public TreePath getPathAtPos(int x, int y) {
+        final TreePath[] path;
+        final boolean result;
+
+        path = new TreePath[1];
+
+        result = GtkTreeView.getPathAtPos(this, x, y, path, null, null, null);
+
+        if (result) {
+            return path[0];
+        } else {
+            return null;
+        }
+    }
+
+    /**
+     * Figure out which TreeViewColumn a given event's co-ordinates correspond
+     * to. See {@link #getPathAtPos(int, int) getPathAtPos()} for a detailed
+     * discussion.
+     * 
+     * <p>
+     * <i>In native GTK, this is implemented as an out parameter on the same
+     * function that powers <code>getPathAtPos()</code>, but we've given it a
+     * more coherent name here.</i>
+     * 
+     * @since 4.0.9
+     */
+    /*
+     * This does not match the completion style we have elsewhere in
+     * java-gnome where we use the return to differentiate the various
+     * out-parameters. That is bad, but diverging from getPathAtPos() to
+     * getAtPosPath() seems very distasteful. This is the less used code path.
+     */
+    public TreeViewColumn getColumnAtPos(int x, int y) {
+        final TreeViewColumn[] column;
+        final boolean result;
+
+        column = new TreeViewColumn[1];
+
+        result = GtkTreeView.getPathAtPos(this, x, y, null, column, null, null);
+
+        if (result) {
+            return column[0];
+        } else {
+            return null;
+        }
+    }
+
+    /**
+     * Get a Rectangle describing the area used by the CellRenderer to draw
+     * the cell at the position described by the row <code>path</code> in the
+     * column <code>column</code>. The Rectangle will be in tree coordinates.
+     * 
+     * <p>
+     * If path points to a row not currently being displayed, the
+     * <code>y</code> and <code>height</code> attributes of the rectangle will
+     * be <code>0</code>.
+     * 
+     * <p>
+     * Note that taking the over a row's Rectangles widths, or over all rows'
+     * heights, would not cover the entire tree; there can be extra pixels of
+     * padding in between rows, etc.
+     * 
+     * <p>
+     * <i>This method is only works if the TreeView has already been
+     * realized.</i>
+     * 
+     * @since 4.0.17
+     */
+    public Rectangle getCellArea(TreePath path, TreeViewColumn column) {
+        final Rectangle rectangle;
+
+        rectangle = new Rectangle(0, 0, 0, 0);
+        GtkTreeView.getCellArea(this, path, column, rectangle);
+
+        return rectangle;
+    }
 }
