@@ -1,13 +1,34 @@
 /*
- * TreeViewColumn.java
+ * java-gnome, a UI library for writing GTK and GNOME programs from Java!
  *
- * Copyright (c) 2007-2009 Operational Dynamics Consulting Pty Ltd, and Others
+ * Copyright © 2007-2010 Operational Dynamics Consulting, Pty Ltd and Others
  *
- * The code in this file, and the library it is a part of, are made available
- * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" plus the "Classpath Exception" (you may link to this code as a
- * library into other programs provided you don't make a derivation of it).
- * See the LICENCE file for the terms governing usage and redistribution.
+ * The code in this file, and the program it is a part of, is made available
+ * to you by its authors as open source software: you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License version
+ * 2 ("GPL") as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GPL for more details.
+ *
+ * You should have received a copy of the GPL along with this program. If not,
+ * see http://www.gnu.org/licenses/. The authors of this program may be
+ * contacted through http://java-gnome.sourceforge.net/.
+ *
+ * Linking this library statically or dynamically with other modules is making
+ * a combined work based on this library. Thus, the terms and conditions of
+ * the GPL cover the whole combination. As a special exception (the
+ * "Claspath Exception"), the copyright holders of this library give you
+ * permission to link this library with independent modules to produce an
+ * executable, regardless of the license terms of these independent modules,
+ * and to copy and distribute the resulting executable under terms of your
+ * choice, provided that you also meet, for each linked independent module,
+ * the terms and conditions of the license of that module. An independent
+ * module is a module which is not derived from or based on this library. If
+ * you modify this library, you may extend the Classpath Exception to your
+ * version of the library, but you are not obligated to do so. If you do not
+ * wish to do so, delete this exception statement from your version.
  */
 package org.gnome.gtk;
 
@@ -73,6 +94,43 @@ public class TreeViewColumn extends Object implements CellLayout
         return GtkTreeViewColumn.getTitle(this);
     }
 
+    /**
+     * Sets the visibility of the TreeViewColumn.
+     */
+    public void setVisible(boolean visible) {
+        GtkTreeViewColumn.setVisible(this, visible);
+    }
+
+    /**
+     * Returns whether the TreeViewColumn is visible or not. If it is visible,
+     * then the TreeView will show the column.
+     * 
+     * @since 4.0.17
+     */
+    public boolean getVisible() {
+        return GtkTreeViewColumn.getVisible(this);
+    }
+
+    /**
+     * Sets the widget in the header to be widget. If widget is null, then the
+     * header button is set with a Label set to the title of tree_column.
+     * 
+     * @since 4.0.17
+     */
+    public void setWidget(Widget widget) {
+        GtkTreeViewColumn.setWidget(this, widget);
+    }
+
+    /**
+     * Returns the Widget in the button on the column header. If a custom
+     * widget has not been set then null is returned.
+     * 
+     * @since 4.0.17
+     */
+    public Widget getWidget() {
+        return GtkTreeViewColumn.getWidget(this);
+    }
+
     /**
      * Set whether the column header allows itself to be clicked. The default
      * is <code>false</code>, not to be clickable, also meaning that header
