@@ -1,12 +1,20 @@
 /*
- * ValidateUtilityMethods.java
+ * java-gnome, a UI library for writing GTK and GNOME programs from Java!
  *
- * Copyright (c) 2007 Operational Dynamics Consulting Pty Ltd
- * 
- * The code in this file, and the suite it is a part of, are made available
- * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" See the LICENCE file for the terms governing usage and
- * redistribution.
+ * Copyright © 2007-2010 Operational Dynamics Consulting, Pty Ltd
+ *
+ * The code in this file, and the program it is a part of, is made available
+ * to you by its authors as open source software: you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License version
+ * 2 ("GPL") as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GPL for more details.
+ *
+ * You should have received a copy of the GPL along with this program. If not,
+ * see http://www.gnu.org/licenses/. The authors of this program may be
+ * contacted through http://java-gnome.sourceforge.net/.
  */
 package com.operationaldynamics.codegen;
 
@@ -26,8 +34,8 @@ import com.operationaldynamics.driver.ImproperDefsFileException;
 public final class ValidateUtilityMethods extends ValidateDefsParsing
 {
     public final void testEncodeJavaNamesToJni() {
-        assertEquals("org_gnome_gtk_GtkButton", Generator.encodeJavaClassName("org.gnome.gtk",
-                "GtkButton"));
+        assertEquals("org_gnome_gtk_GtkButton",
+                Generator.encodeJavaClassName("org.gnome.gtk", "GtkButton"));
         assertEquals("gtk_1button_1set_1label", Generator.encodeJavaMethodName("gtk_button_set_label"));
         assertEquals("setLabelTheRightWay", Generator.toCamel("set_label_the_right_way"));
     }
@@ -65,8 +73,12 @@ public final class ValidateUtilityMethods extends ValidateDefsParsing
         assertEquals("(Widget) objectFor(b)", ot.translationToJava("b", context));
     }
 
+    public final void testConsantNameMunging() {
+        assertEquals("REALLY_IMPORTANT", Generator.toAllCaps("really-important"));
+    }
+
     public final void testSignalNameMunging() {
-        assertEquals("DELETE_EVENT", Generator.toAllCaps("delete-event"));
+        assertEquals("DeleteEvent", Generator.toPascalCase("delete_event"));
     }
 
     public final void testPascalCaseBackToUnderscores() {
